"use strict";

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource:///modules/CustomizableUI.jsm");
Cu.import("resource://gre/modules/Services.jsm");


function addUI(window) {
    let document = window.document;
    let menu = document.getElementById("menu_ToolsPopup");
    if (!menu) {
        return;
    }
    let menuitem = document.createElement("menuitem");
    menuitem.setAttribute("id", "speedtrans_menu");
    menuitem.setAttribute("label", "Pad.ma SpeedTrans");
    menuitem.addEventListener("click", openSpeedTrans);
    menuitem.addEventListener("command", openSpeedTrans);
    menu.appendChild(menuitem);
}

function removeUI(window) {
    let document = window.document;
    let item = document.getElementById("speedtrans_menu");
    if (item) {
        item.parentNode.removeChild(item);
    }
}

function install(aData, aReason) {
}
function uninstall(aData, aReason) {
}

function startup(aData, aReason) {
    // the 'style' directive isn't supported in chrome.manifest for boostrapped
    // extensions, so this is the manual way of doing the same.
    let io = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
    let css = Cc["@mozilla.org/content/style-sheet-service;1"].getService(Ci.nsIStyleSheetService);
    let uri = io.newURI("chrome://speedtrans/content/toolbar.css", null, null);
    css.loadAndRegisterSheet(uri, css.USER_SHEET);

    CustomizableUI.createWidget({
        id: "speedtrans-toolbarbutton",
        defaultArea: CustomizableUI.AREA_PANEL,
        allowedAreas: [CustomizableUI.AREA_PANEL, CustomizableUI.AREA_NAVBAR],
        removable: true,
        label: "SpeedTrans",
        tooltiptext: "Open SpeedTrans",
        onClick: openSpeedTrans
    });
    watchWindows(addUI);
}

function shutdown() {
    watchWindows(removeUI, "enumerate-only");
}

function openSpeedTrans(aEvent) {
    let gBrowser= aEvent.target.ownerDocument.defaultView.gBrowser;
    gBrowser.selectedTab = gBrowser.addTab('chrome://speedtrans/content/transcribe.html');
}

/**
 * Apply a callback to each open and new browser windows.
 *
 * @usage watchWindows(callback): Apply a callback to each browser window.
 * @param [function] callback: 1-parameter function that gets a browser window.
 * @param [string] option: If set to "enumerate-only", don't add a listener
 *   for future window-opens.
 */
function watchWindows(callback, option) {
  // Wrap the callback in a function that ignores failures
  function watcher(window) {
    //try {
      // Now that the window has loaded, only handle browser windows
      let {documentElement} = window.document;
      if (documentElement.getAttribute("windowtype") == "navigator:browser")
        callback(window);
    //}
    //catch(ex) {}
  }

  // Wait for the window to finish loading before running the callback
  function runOnLoad(window) {
    // Listen for one load event before checking the window type
    window.addEventListener("load", function runOnce() {
      window.removeEventListener("load", runOnce, false);
      watcher(window);
    }, false);
  }

  // Add functionality to existing windows
  let windows = Services.wm.getEnumerator(null);
  while (windows.hasMoreElements()) {
    // Only run the watcher immediately if the window is completely loaded
    let window = windows.getNext();
    if (window.document.readyState == "complete")
      watcher(window);
    // Wait for the window to load before continuing
    else
      runOnLoad(window);
  }

  // Watch for new browser windows opening then wait for it to load
  function windowWatcher(subject, topic) {
    if (topic == "domwindowopened")
      runOnLoad(subject);
  }
  if (option != "just-enumerate") {
    Services.ww.registerNotification(windowWatcher);
    // Make sure to stop watching for windows if we're unloading
    unload(function() Services.ww.unregisterNotification(windowWatcher));
  }
}

/**
 * Save callbacks to run when unloading. Optionally scope the callback to a
 * container, e.g., window. Provide a way to run all the callbacks.
 *
 * @usage unload(): Run all callbacks and release them.
 *
 * @usage unload(callback): Add a callback to run on unload.
 * @param [function] callback: 0-parameter function to call on unload.
 * @return [function]: A 0-parameter function that undoes adding the callback.
 *
 * @usage unload(callback, container) Add a scoped callback to run on unload.
 * @param [function] callback: 0-parameter function to call on unload.
 * @param [node] container: Remove the callback when this container unloads.
 * @return [function]: A 0-parameter function that undoes adding the callback.
 */
function unload(callback, container) {
  // Initialize the array of unloaders on the first usage
  let unloaders = unload.unloaders;
  if (unloaders == null)
    unloaders = unload.unloaders = [];

  // Calling with no arguments runs all the unloader callbacks
  if (callback == null) {
    unloaders.slice().forEach(function(unloader) unloader());
    unloaders.length = 0;
    return;
  }

  // The callback is bound to the lifetime of the container if we have one
  if (container != null) {
    // Remove the unloader when the container unloads
    container.addEventListener("unload", removeUnloader, false);

    // Wrap the callback to additionally remove the unload listener
    let origCallback = callback;
    callback = function() {
      container.removeEventListener("unload", removeUnloader, false);
      origCallback();
    }
  }

  // Wrap the callback in a function that ignores failures
  function unloader() {
    try {
      callback();
    }
    catch(ex) {}
  }
  unloaders.push(unloader);

  // Provide a way to remove the unloader
  function removeUnloader() {
    let index = unloaders.indexOf(unloader);
    if (index != -1)
      unloaders.splice(index, 1);
  }
  return removeUnloader;
}
